# 11.3 亚马逊雨林地图可视化

在上一节中，已经知道了每个区域在这将近 20 年里总共发生过多少起火灾。那么我们能不能更加直观的感受一下每年每个区域的火灾次数的变化情况呢？此时你可能会想到这样的代码：

```python
data[['year','state','number']].groupby(['year','state']).number.sum()
```

然后就会看到这样的结果。

![](8.jpg)


没错，我们需要的数据就是这些，但是我们是感官动物，看到一堆数字时并不觉得这堆数字有多棒。所以我们接下来就尝试将这些数据进行可视化，让我们能从可视化结果中看出每年这些区域的火灾次数的变化情况。

首先，我们需要一张巴西的地图，这张地图在网络上有现成的，可以从  https://geodata.lib.berkeley.edu/catalog/stanford-ys298mq8577 获取。下载好地图的压缩包后，可以将后缀名为 .shp 的文件解压出来，然后使用如下代码就可以将地图画出来。

```python
# 加载geopandas
import geopandas as gp

# 读取解压好的.shp文件
geo_data = gp.read_file('./geo_brazil_data.shp')
# .shp文件中的nome字段表示state字段，所以需要字段重命名
geo_data = geo_data.rename(columns = {'nome': 'state'})
# 画图
geo_data.plot()
```

![](9.jpg)

嗯，看上去是不是有点样子了？不过这还不算什么，因为这仅仅是巴西的地图轮廓而已。我们的目标是每年的火灾变化情况反应到这张地图上。所以可以编写如下代码来实现该功能：

```python
# 每年每个区域的火灾次数统计
ani_d2 = data[['year','state','number']].groupby(['year','state']).number.sum().reset_index()

# 构建透视表
ani_d2_pivot = ani_d2.pivot_table(values='number', index=['state'], columns='year').reset_index()
ani_d2_final = geo_data.set_index('state').join(ani_d2_pivot.set_index('state'))

# 火灾次数的最小和最大值
vmin, vmax = 200, 400

for year in ani_d2_final.columns[1:]:
    # 创建图
    fig = ani_d2_final.plot(column=year, cmap='Oranges', figsize=(8,5), linewidth=0.8, edgecolor='0.8', vmin=vmin, vmax=vmax, legend=True, norm=plt.Normalize(vmin=vmin, vmax=vmax))

    # 关闭轴
    fig.axis('off')

    # 设置图的标题
    fig.set_title('Fire In Brazil', fontdict={'fontsize': '15', 'fontweight' : '3'})
    
    # 图的注释
    fig.annotate(year, xy=(0.1, .225), xycoords='figure fraction', horizontalalignment='left', verticalalignment='top',fontsize=20)
    chart = fig.get_figure()
    # 按年份保存图片
    chart.savefig(str(year) +'_fire.png', dpi=300)
    plt.close()
```

当整段代码执行完毕后，我们会在当前目录下看到 20 张 .png 图片，图片如下：

![](10.jpg)

![](11.jpg)

![](12.jpg)

![](13.jpg)

![](14.jpg)

![](15.jpg)

![](16.jpg)

![](17.jpg)

![](18.jpg)

![](19.jpg)

![](20.jpg)

![](21.jpg)

![](22.jpg)

![](23.jpg)

![](24.jpg)

![](25.jpg)

![](26.jpg)

![](27.jpg)

![](28.jpg)

![](29.jpg)

可以看到，火灾的重灾区的火灾发生次数比较稳定。不过如果将这些图片合成为一张 gif 动图，可视化的效果就更加明显。那怎样将这图片转换为动图，就作为一个练习留给你练手了。