# 11.2 使用图表来探索亚马逊雨林

现在我们不妨使用 python 代码画出每年亚马逊雨林火灾发生次数的折线图。

```python
# 导入matplotlib中的MaxNLocator和FuncFormatter
from matplotlib.pyplot import MaxNLocator, FuncFormatter

# 设置折线图的大小
plt.figure(figsize=(12,4))

# 绘制折线图，横轴为年份，纵轴为火灾发生次数
ax = sns.lineplot(x = 'year', y = 'number', data = year_mo_state, estimator = 'sum', color = 'orange', lw = 3, 
                  err_style = None)

# 设置折线图的标题
plt.title('Total Fires in Brazil : 1998 - 2017', fontsize = 18)
# 设置横轴与纵轴的名称
plt.xlabel('Year', fontsize = 14)
plt.ylabel('Number of Fires', fontsize = 14)

# 设置刻度
ax.xaxis.set_major_locator(plt.MaxNLocator(19))
ax.set_xlim(1998, 2017)

# 设置日期格式
ax.get_yaxis().set_major_formatter(plt.FuncFormatter(lambda x, p: format(int(x), ',')))
```

![](4.jpg)

从图表可以看出，火灾在过去的 20 年中急剧增加，从 1998 年的 2 万起增加到 2017 年的 36000，将近一倍！然后，心细观察的话不难发现，火灾有不断增长的趋势，因此我们可以预计在接下来的几年里还会发生更多的火灾！

看完每年的火灾发生次数，我们再来看看每月的火灾发生次数，看看发生火灾与月份有没有什么关系。

```python
# 设置折线图的大小
plt.figure(figsize=(12,4))

# 绘制盒图，横轴是月份，纵轴是火灾发生次数
sns.boxplot(x = 'month', order = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep','Oct', 'Nov', 'Dec'], 
            y = 'number', data = year_mo_state)

# 设置盒的标题和横纵轴的名称
plt.title('Fires in Brazil by Month', fontsize = 18)
plt.xlabel('Month', fontsize = 14)
plt.ylabel('Number of Fires', fontsize = 14)
```

![](5.jpg)

该盒图仿佛暗示着，每年的下半年，火灾发生的概率更高，尤其是从夏天到秋天这段时间。这也是符合常理的，毕竟气候干燥，容易发生火灾。

那么火灾的发生与区域有关吗？我们同样可以统计一下。

```python
# 统计火灾发生次数最高的10个区域
data.groupby(by = 'state')['number'].sum().sort_values(ascending = False).head(10)
```

![](6.jpg)

可以看到，火灾重灾区的第二名和第三名非常接近，但是第一名比第二名高出了几乎 400000 次！嗯，我们可以网上搜索一下巴西的地图，看看这些重灾区的位置。

![](7.jpg)

第一名重灾区在巴西的中部，如果发生火灾，后果可想而知。为什么 Mato Grosso 会排在榜首呢？那是因为 Mato Grosso 是巴西第三大州。这个州人口占巴西总人口的比重很小，约为 1.5 %，但农业产业非常强大。

过去，Mato Grosso 是巴西最大的二氧化碳排放州之一，这是由于其强大的农业经济驱动下的森林火灾和森林砍伐所致。


