# 5.2 线性回归算法原理

## 什么是线性回归

线性回归是什么意思？可以拆字释义。回归肯定不用我多说了，那什么是线性呢？我们可以回忆一下初中 时学过的直线方程：`y=k*x+b`

这个式子表达的是，当我知道 k （参数）和 b （参数）的情况下，我随便给一个 x 我都能通过这个方程 算出 y 来。而且呢，这个式子是线性的，为什么呢？因为从直觉上来说，你都知道，这个式子的函数图像 是条直线。 

从理论上来说，这式子满足线性系统的性质。你可能会觉得疑惑，这一节要说的是线性回归，我说个这么 low 直线方程干啥？其实，说白了，线性回归就是在 N 维空间中找一个形式像直线方程一样的函数来拟合数据而已。比如说，我 现在有这么一张图，横坐标代表房子的面积，纵坐标代表房价。

![](2.jpg)

然后呢，线性回归就是要找一条直线，并且让这条直线尽可能地拟合图中的数据点。

那如果让 1000 位朋友来找这条直线就可能找出 1000 种直线来，比如这样

![](4.jpg)

这样

![](5.jpg)

或者这样

![](6.jpg)

喏，其实找直线的过程就是在做线性回归，只不过这个叫法更有高大上而已。

## 损失函数

那既然是找直线，那肯定是要有一个评判的标准，来评判哪条直线才是最好的。道理我们都懂，那咋评判呢？其实只要算一下实际房价和我找出的直线根据房子大小预测出来的房价之间的差距就行了。说白了就 是算两点的距离。当把所有实际房价和预测出来的房价的差距（距离）算出来然后做个加和，就能量化出 现在预测的房价和实际房价之间的误差。例如下图中我画了很多条小数线，每一条小数线就是实际房价和预测房价的差距（距离）。

![](7.jpg)

然后把每条小竖线的长度加起来就等于现在通过这条直线预测出的房价与实际房价之间的差距。那每条小竖线的长度的加和怎么算？其实就是欧式距离加和，公式为：$$\sum_{i=1}^m(y^{(i)}-\hat y{^{(i)}})^2$$(其中$$y(i)$$表示的是实际房价，$$\hat y (i)$$表示的是预测房价)。


这个欧氏距离加和其实就是用来量化预测结果和真实结果的误差的一个函数。在机器学习中称它为**损失函数**（说白了就是计算误差的函数）。那有了这个函数，就相当于有了一个评判标准，当这个函数的值越小，就越说明找到的这条直线越能拟合房价数据。所以说啊，线性回归就是通过这个损失函数做为评判标准来找出一条直线。

如果假设$$h_{(\theta)}(x)$$表示当权重为$$\theta$$，输入为$$x$$时计算出来的$$ \hat y(i)$$，那么线性回归的损失函数$$J(\theta)$$就是：

<center>
$$
J(\theta)=\frac{1}{2}\sum^m_{i=1}(h_\theta(x^i)-y^i)^2
$$
</center>

## 怎样计算出线性回归的解？

现在你应该已经弄明白了一个事实，那就是我只要找到一组参数（也就是线性方程每一项上的系数）能让我的损失函数的值最小，那我这一组参数就能最好的拟合我现在的训练数据。

那怎么来找到这一组参数呢？其实我们可以使用线性回归的**正规方程解**，这名字听起来高大上，其实本质就是根据一个固定的式子计算出参数。线性回归的正规方程解是这样算出来的。

假设数据中`m`条记录，每条记录中有`n`个特征，可以使用矩阵的表示方法，预测函数可以写为：

$$
Y = X.W
$$

其损失函数可以表示为

$$
loss = \frac{1}{m}(Y-X.\theta)^T(Y-X.\theta)
$$

其中，标签`Y`为`m`行`1`列的矩阵，特征`X`为`m`行`(n+1)`列的矩阵，回归系数\theta为`(n+1)`行`1`列的矩阵，对\theta求导，并令其导数为零可解得：

$$
W=(\theta^T\theta)^{-1}\theta^TY
$$

这个就是正规方程解，我们可以通过正规方程解直接求得我们所需要的参数。