# 8.2 Apriori算法原理

## Apriori原理

假设我们在经营一家商品种类并不多的小卖铺，我们对那些经常在一起被购买的商品非常感兴趣。我们只有 4 种商品：薯片，可乐，纸巾和电池。那么所有可能被一起购买的商品组合都有哪些？这些商品组合可能只有一种商品，比如薯片, 也可能包括两种、三种或者所有四种商品。我们并不关心某人买了两包薯片以及四对电池的情况，我们只关心他购买了一种或多种商品。所以这四种商品的组合图如下(其中用 0 表示薯片，1 表示可乐，2 表示纸巾， 3 表示电视)：

![](2.jpg)


上一节说过，我们的目标是找到经常在一起购买的物品集合，并且我们通常会使用集合的支持度来度量其出现的频率。一个集合的支持度是指有多少比例的交易记录包含该集合。如何对一个给定的集合，比如`{0，3}`，来计算其支持度？很显然，可以遍历毎条记录并检查该记录包含 0 和 3  ，如果记录确实同时包含这两项，那么就增加总计数值。在扫描完所有数据之后，使用统计得到的总数除以总的交易记录数，就可以得到支持度。这个计算过程还仅仅是对`{0, 3}`这个项集而言，如果想要将海量数据中所有可能出现的项集的支持度给算出来的话，那么很用可能算一辈子也算不完。


为了降低所需的计算时间，研究人员发现一种所谓的 Apriori 原理。Apriori 原理可以帮我们减少可能感兴趣的项集。Apriori 原理是说如果某个项集是频繁的，那么它的所有子集也是频繁的。

对于我们开小卖铺的例子，如果`{0, 1}`这个项集是频繁的，那么`{0}`和`{1}`也一定是频繁的。如果将这个命题进行逆否处理，那么会得到另一个结论，就是如果一个项集是非频繁的，那么它的所有超集都是非频繁的。

那这个结论有什么用呢？假设`{2, 3}`这个项集是非频繁的，那么跟它有关的`{0, 2, 3}`，`{1, 2, 3}`，`{0, 1, 2, 3}`都是非频繁的。也就是说算完`{2, 3}`的支持度发现不是频繁的，那么后面 3 个项集的支持度就不需要算了，这样就减小了计算的时间复杂度。

![](3.jpg)


## Apriori算法流程

Apriori 算法的两个输人参数分别是最小支持度和数据集。该算法首先会生成所有单个物品的项集列表。接着扫描交易记录来查看哪些项集满足最小支持度要求，那些不满足最小支持度的集合会被去掉。然后，对剩下来的集合进行组合以生成包含两个元素的项集。接下来，再重新扫描交易记录，去掉不满足最小支持度的项集。该过程重复进行直到所有项集都被去掉。


所以 Apriori 算法的伪代码如下：

```python
while 集合中的项的个数 > 0:
    构建一个由 k 个项组成的候选项集的列表
    确认每个项集都是频繁的
    保留频繁项集并构建 k+1 项组成的候选项集的列表
```

## 从频繁项集中挖掘关联规则

要找到关联规则，需要从一个频繁项集开始。我们知道集合中的元素是不重复的，但我们想知道基于这些元素能否获得其他内容。例如某个元素或者某个元素集合可能会推导出另一个元素 。从小卖铺的例子可以得到，如果有一个频繁项集`{薯片，西瓜}`，那么就可能有一条关联规
则`薯片->西瓜`。这意味着如果有人购买了薯片，那么在统计上他会购买西瓜的概率较大。

但是，这一条反过来并不总是成立。也就是说，即使`薯片->西瓜`统计上显著，那么`薯片->西瓜`也不一定成立。(从逻辑研究上来讲，箭头左边的集合称作前件，箭头右边的集合称为后件。)

那么怎样挖掘关联规则呢？在发现频繁项集时我们发现的是高于最小支持度的频繁项集，对于关联规则，也是用这种类似的方法。以小卖铺的例子为例，从项集`{0, 1, 2, 3}`产生的关联规则中，找出可信度高于最小可信度的关联规则。(**PS:Apriori 原理对于关联规则同样适用。**)





